@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -81,6 +81,18 @@ only_spaces (const char *text)
 }
 
 
+static void
+add_files_done_cb (gpointer data)
+{
+	FRWindow *window = data;
+
+	window_pop_message (window);
+
+	visit_dir_handle_free (window->vd_handle);
+	window->vd_handle = NULL;
+}
+
+
 static int
 open_file_ok_cb (GtkWidget *w,
                  GtkWidget *file_sel)
@@ -159,12 +171,16 @@ open_file_ok_cb (GtkWidget *w,
 
 			first_path = g_build_path (G_DIR_SEPARATOR_S, base_dir, first, NULL);
 			if (path_is_dir (first_path)) {
-				fr_archive_add_directory (window->archive, 
-							  file_name_from_path (first), 
-							  base_dir, 
-							  FALSE,
-							  window->password,
-							  window->compression);
+				window_push_message (window, _("Getting file list, wait please..."));
+				window->vd_handle = fr_archive_add_directory (
+				      window->archive, 
+				      file_name_from_path (first), 
+				      base_dir, 
+				      FALSE,
+				      window->password,
+				      window->compression,
+				      add_files_done_cb,
+				      window);
 			} else 
 				fr_archive_add (window->archive,
 						files,
@@ -201,19 +217,23 @@ open_file_ok_cb (GtkWidget *w,
 		} else
 			exclude_files = NULL;
 
-		fr_archive_add_with_wildcard (window->archive,
-					      file_name_from_path (path),
-					      exclude_files,
-					      base_dir,
-					      update,
-					      recursive,
-					      ! no_symlinks,
-					      same_fs,
-					      no_backup_files,
-					      no_dot_files,
-					      ignore_case,
-					      window->password,
-					      window->compression);
+		window_push_message (window, _("Getting file list, wait please..."));
+		window->vd_handle = fr_archive_add_with_wildcard (
+			  window->archive,
+			  file_name_from_path (path),
+			  exclude_files,
+			  base_dir,
+			  update,
+			  recursive,
+			  ! no_symlinks,
+			  same_fs,
+			  no_backup_files,
+			  no_dot_files,
+			  ignore_case,
+			  window->password,
+			  window->compression,
+			  add_files_done_cb,
+			  window);
 	}
 
 	g_free (path);
