@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #ifndef FILE_UTILS_H
 #define FILE_UTILS_H
 
-#include <sys/types.h>
 #include <time.h>
 #include <libgnomevfs/gnome-vfs-file-size.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -81,6 +80,9 @@ gboolean            match_patterns               (char       **patterns,
 
 char **             search_util_get_patterns     (const char  *pattern_string);
 
+GnomeVFSFileSize    get_dest_free_space          (const char  *path);
+
+
 #define get_home_relative_dir(x)        \
         g_strconcat (g_get_home_dir (), \
                      "/",               \
