@@ -116,7 +116,8 @@ FileTypeDescription type_desc[] = {
 	{ N_("Tar compressed with lzop (.tar.lzo)"),   ".tar.lzo" }, 
 	{ N_("Tar compressed with compress (.tar.Z)"), ".tar.Z" }, 
 	{ N_("War (.war)"),                            ".war" },
-	{ N_("Zip (.zip)"),                            ".zip" }
+	{ N_("Zip (.zip)"),                            ".zip" },
+	{ N_("Zoo (.zoo)"),                            ".zoo" }
 };
 
 static int type_desc_n = sizeof (type_desc) / sizeof (FileTypeDescription);
@@ -393,28 +394,24 @@ copy_archive_ok_cb (GtkWidget    *w,
 {
 	FRWindow   *window;
 	const char *path;
-	const char *filename;
-	char       *folder;
-	char       *new_path;
+	char       *new_filename;
 
 	gtk_widget_hide (data->file_sel);
 
 	window = data->window;
 	path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (data->file_sel));
 
-	if ((path == NULL) || (*path == 0)) {
+	if (path == NULL) {
                 gtk_widget_destroy (data->file_sel);
 		return;
 	}
 
-	filename = file_name_from_path (path);
-	if ((filename == NULL) || (*filename == 0))
-		filename = file_name_from_path (window->archive_filename);
-	folder = remove_level_from_path (path);
-	new_path = g_build_filename (folder, filename, NULL);
-	g_free (folder);
+	if (path[strlen (path) - 1] == '/')
+		new_filename = g_strconcat (path, file_name_from_path (window->archive_filename), NULL);
+	else
+		new_filename = g_strconcat (path, "/", file_name_from_path (window->archive_filename), NULL);
 
-	if (path_is_file (new_path)) {
+	if (path_is_file (new_filename)) {
 		GtkWidget *d;
 		int        r;
 
@@ -430,7 +427,7 @@ copy_archive_ok_cb (GtkWidget    *w,
 		gtk_widget_destroy (GTK_WIDGET (d));
 		
 		if (r != GTK_RESPONSE_YES) {
-			g_free (new_path);
+			g_free (new_filename);
 			gtk_widget_destroy (data->file_sel);
 			return;
 		}
@@ -438,9 +435,9 @@ copy_archive_ok_cb (GtkWidget    *w,
 	
 	if (data->copy) {
 #ifdef DEBUG
-		g_print ("cp %s %s\n", window->archive_filename, new_path);
+		g_print ("cp %s %s\n", window->archive_filename, new_filename);
 #endif
-		if (! file_copy (window->archive_filename, new_path)) {
+		if (! file_copy (window->archive_filename, new_filename)) {
 			GtkWidget *dialog;
 			dialog = gtk_message_dialog_new (GTK_WINDOW (window->app),
 							 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -452,10 +449,10 @@ copy_archive_ok_cb (GtkWidget    *w,
 		}
 	} else {
 #ifdef DEBUG
-		g_print ("mv %s %s\n", window->archive_filename, new_path);
+		g_print ("mv %s %s\n", window->archive_filename, new_filename);
 #endif
-		if (file_move (window->archive_filename, new_path)) 
-			window_archive_rename (window, new_path);
+		if (file_move (window->archive_filename, new_filename)) 
+			window_archive_rename (window, new_filename);
 		else {
 			GtkWidget *dialog;
 			dialog = gtk_message_dialog_new (GTK_WINDOW (window->app),
@@ -468,7 +465,7 @@ copy_archive_ok_cb (GtkWidget    *w,
 		}
 	}
 
-	g_free (new_path);
+	g_free (new_filename);
 	gtk_widget_destroy (data->file_sel);
 }
 
@@ -779,9 +776,7 @@ about_cb (GtkWidget *widget,
 		"Paolo Bacchilega <paolo.bacch@tin.it>", NULL
 	};
 	const char       *documenters [] = {
-		"Alexander Kirillov", 
-		"Breda McColgan",
-		NULL
+		"Alexander Kirillov", NULL
 	};
 	const char       *translator_credits = _("translator_credits");
 
@@ -793,7 +788,7 @@ about_cb (GtkWidget *widget,
 	logo = gdk_pixbuf_new_from_file (PIXMAPSDIR "/file-roller.png", NULL);
 	about = gnome_about_new (_("File Roller"), 
 				 VERSION,
-				 "Copyright \xc2\xa9 2001-2002 Free Software Foundation, Inc.",
+				 "Copyright \xc2\xa9 2001-2003 Free Software Foundation, Inc.",
 				 _("An archive manager for GNOME."),
 				 authors,
 				 documenters,
@@ -903,16 +898,39 @@ sort_list_reversed (GtkWidget *widget,
 }
 
 
+/* -- stop_cb -- */
+
+
 void
-stop_cb (GtkWidget *widget, 
-	 void *data)
+stop__step2 (gpointer data)
 {
 	FRWindow *window = data;
+
 	if (window->activity_ref > 0)
 		fr_process_stop (window->archive->process);
 }
 
 
+void
+stop_cb (GtkWidget *widget, 
+	 void      *data)
+{
+	FRWindow *window = data;
+
+	if (window->vd_handle != NULL) {
+		visit_dir_async_interrupt (window->vd_handle, 
+					   stop__step2, 
+					   window);
+		window->vd_handle = NULL;
+
+	} else
+		stop__step2 (window);
+}
+
+
+/**/
+
+
 void
 reload_cb (GtkWidget *widget, 
 	   void *data)
